// The MIT License
// 
// Copyright (c) 2011 Votizen Inc.
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

var nest        = require('nest'),
    util        = require('util'),
    http        = require('http'),
    querystring = require('querystring');

/**
 * Client constructor.
 *
 * @constructor
 * @param {Object} options
 * @extends nest.Client
 */
var Client = exports.Client = function (options) {
  options = options || {};

  nest.Client.call(this, {
    host:       'api.crowdflower.com',
    secure:     false,
    path:       '/' + (options.version || 'v1') + '/',
    response:   'json',
    type:       'json',
    params: {
      key:      options.key
    },
    headers: {
      "Accept": 'application/json'
    }
  });
};

// Inherits from nest
util.inherits(Client, nest.Client);

/**
 * Alternative constructor.
 */
exports.createClient = function (options) {
  return new Client(options);
};

/**
 * Override the nest _request to add relevant logic to responses.
 *
 * @param {Object} options: A hash of options for the request.
 * @param {Function} callback
 * @private
 */
Client.prototype._request = function (method, options, callback) {
  callback = callback || function () {};

  return nest.Client.prototype._request.call(this, method, options, function (error, response, data) {
    if (error) {
      return callback(error);
    } else if (data.message && data.message.error) {
      return callback(new Error(data.message.error), data);
    } else if (data.error && data.error.message) {
      return callback(new Error(data.error.message), data);
    }

    callback(null, data);
  });
};

/**
 * Setup a Webhook listening server.
 * Events get emitted on the server object.
 *
 * @param {Object} options
 */
Client.prototype.webhook = function (options) {
  var server = http.createServer(function (request, response) {
    var body = '';

    if (options.path === request.url) {
      request.on('data', function (data) {
        body += data;
      });
      request.on('end', function () {
        try {
          body = querystring.parse(body);
          server.emit(body.signal, JSON.parse(body.payload));
        } catch (error) {}

        response.writeHead(200, {
          "Server": "node.js"
        });
        response.end();
      });
    } else {
      body = '404 Not found.';
      response.writeHead(404, {
        "Server":         "node.js",
        "Content-Length": body.length,
        "Content-Type":   "text/plain"
      });
      response.end(body);
    }
  });

  server.listen(options.port || 8080, options.host || null);

  return server;
};
